"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getPackageEntryPoints = getPackageEntryPoints;

var _getPackageMetadata = _interopRequireDefault(require("./getPackageMetadata"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

async function getPackageEntryPoints(packagePath) {
  const {
    packageJsonsByPackagePath
  } = await (0, _getPackageMetadata.default)();
  const packageJson = packageJsonsByPackagePath[packagePath];
  let compilingBin = false;
  let main;

  if (packageJson.main) {
    main = packageJson.main;
  } else {
    if (packageJson.bin) {
      const bins = Object.values(packageJson.bin);

      if (bins.length === 1) {
        compilingBin = true;
        main = bins[0];
      } else {
        throw new Error(`package.json at ${packagePath} contains multiple "bin" values, bailing...`);
      }
    } else {
      throw new Error(`package.json at ${packagePath} does not have a "main" or "bin" field, bailing...`);
    }
  }

  return {
    main,
    compilingBin
  };
}