"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.buildPackage = buildPackage;

var _util = require("util");

var _rimraf2 = _interopRequireDefault(require("rimraf"));

var path = _interopRequireWildcard(require("path"));

var _tar = require("tar");

var _execa = _interopRequireDefault(require("execa"));

var _changeCase = require("change-case");

var fse = _interopRequireWildcard(require("fs-extra"));

var _getLogger = require("./getLogger");

var _getPackageEntryPoints = require("./getPackageEntryPoints");

var _getPackageMetadata = _interopRequireDefault(require("./getPackageMetadata"));

var _getModularRoot = _interopRequireDefault(require("../utils/getModularRoot"));

var _makeBundle = require("./makeBundle");

var _makeTypings = require("./makeTypings");

var _getRelativeLocation = _interopRequireDefault(require("../utils/getRelativeLocation"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// While working on this file, be aware that builds
// could be happening simultaneously across packages, so
// try be 'thread-safe'. No state outside of functions
// shorthand for building every workspace, if you're ever debugging this flow
// rm -rf dist && yarn modular build `ls -m1 packages | sed -e 'H;${x;s/\n/,/g;s/^,//;p;};d'`
const outputDirectory = 'dist';
const rimraf = (0, _util.promisify)(_rimraf2.default);

async function buildPackage(target, preserveModules = false) {
  const modularRoot = (0, _getModularRoot.default)();
  const packagePath = await (0, _getRelativeLocation.default)(target);
  const {
    publicPackageJsons
  } = await (0, _getPackageMetadata.default)();

  if (process.cwd() !== modularRoot) {
    throw new Error('This command can only be run from the root of a modular project');
  }

  const logger = (0, _getLogger.getLogger)(packagePath); // ensure the root build folder is ready

  await fse.mkdirp(outputDirectory); // delete any existing local build folders

  await rimraf(path.join(modularRoot, packagePath, `${outputDirectory}-cjs`));
  await rimraf(path.join(modularRoot, packagePath, `${outputDirectory}-es`));
  await rimraf(path.join(modularRoot, packagePath, `${outputDirectory}-types`)); // Generate the typings for a package first so that we can do type checking and don't waste time bundling otherwise

  const {
    compilingBin
  } = await (0, _getPackageEntryPoints.getPackageEntryPoints)(packagePath);

  if (!compilingBin) {
    await (0, _makeTypings.makeTypings)(packagePath);
  } // generate the js files now that we know we have a valid package


  const didBundle = await (0, _makeBundle.makeBundle)(packagePath, preserveModules);

  if (!didBundle) {
    return;
  }

  const originalPkgJsonContent = await fse.readJson(path.join(modularRoot, packagePath, 'package.json'));
  const packageName = originalPkgJsonContent.name; // switch in the special package.json

  try {
    const publicPackageJson = publicPackageJsons[packageName];
    await fse.writeJson(path.join(packagePath, 'package.json'), publicPackageJson, {
      spaces: 2
    });
    await (0, _execa.default)('yarnpkg', // TODO: verify this works on windows
    ['pack', '--silent', '--filename', path.join(modularRoot, outputDirectory, (0, _changeCase.paramCase)(packageName) + '.tgz')], {
      cwd: path.join(modularRoot, packagePath),
      stdin: process.stdin,
      stderr: process.stderr,
      stdout: process.stdout
    });
  } finally {
    // now revert package.json
    await fse.writeJson(path.join(modularRoot, packagePath, 'package.json'), originalPkgJsonContent, {
      spaces: 2
    });
  } // cool. now unpack the tgz's contents in the root dist


  await fse.mkdirp(path.join(outputDirectory, (0, _changeCase.paramCase)(packageName)));
  await (0, _tar.extract)({
    file: path.join(outputDirectory, (0, _changeCase.paramCase)(packageName) + '.tgz'),
    strip: 1,
    C: path.join(outputDirectory, (0, _changeCase.paramCase)(packageName))
  }); // (if you're curious why we unpack it here, it's because
  // we observed problems with publishing tgz files directly to npm.)
  // delete the local dist folders

  await rimraf(path.join(modularRoot, packagePath, `${outputDirectory}-cjs`));
  await rimraf(path.join(modularRoot, packagePath, `${outputDirectory}-es`));
  await rimraf(path.join(modularRoot, packagePath, `${outputDirectory}-types`)); // then delete the tgz

  await fse.remove(path.join(modularRoot, outputDirectory, (0, _changeCase.paramCase)(packageName) + '.tgz')); /// and... that's it

  logger.log('finished');
}