"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.makeTypings = makeTypings;

var path = _interopRequireWildcard(require("path"));

var ts = _interopRequireWildcard(require("typescript"));

var fse = _interopRequireWildcard(require("fs-extra"));

var _getLogger = require("./getLogger");

var _reportTSDiagnostics = require("./reportTSDiagnostics");

var _getPackageMetadata = _interopRequireDefault(require("./getPackageMetadata"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const outputDirectory = 'dist';
const typescriptConfigFilename = 'tsconfig.json';

async function makeTypings(packagePath) {
  const logger = (0, _getLogger.getLogger)(packagePath);
  const {
    typescriptConfig
  } = await (0, _getPackageMetadata.default)();
  logger.log('generating .d.ts files'); // make a shallow copy of the configuration

  const tsconfig = { ...typescriptConfig,
    compilerOptions: { ...typescriptConfig.compilerOptions
    }
  }; // then add our custom stuff
  // Only include src files from the package to prevent already built
  // files from interferring with the compile

  tsconfig.include = [`${packagePath}/src`];
  tsconfig.compilerOptions = { ...tsconfig.compilerOptions,
    declarationDir: `${packagePath}/${outputDirectory}-types`,
    rootDir: packagePath
  }; // Extract config information

  const configParseResult = ts.parseJsonConfigFileContent(tsconfig, ts.sys, path.dirname(typescriptConfigFilename));

  if (configParseResult.errors.length > 0) {
    (0, _reportTSDiagnostics.reportTSDiagnostics)(packagePath, configParseResult.errors);
    throw new Error('Could not parse Typescript configuration');
  }

  const host = ts.createCompilerHost(configParseResult.options);

  host.writeFile = (fileName, contents) => {
    fse.mkdirpSync(path.dirname(fileName));
    fse.writeFileSync(fileName, contents);
  }; // Compile


  const program = ts.createProgram(configParseResult.fileNames, configParseResult.options, host);
  const emitResult = program.emit(); // Report errors

  const diagnostics = ts.getPreEmitDiagnostics(program).concat(emitResult.diagnostics);

  if (diagnostics.length > 0) {
    (0, _reportTSDiagnostics.reportTSDiagnostics)(packagePath, diagnostics);
    throw new Error('Could not generate .d.ts files');
  }
}